private data class Move(val e1: Pair<Int, Int>, val e2: Pair<Int, Int>) {
    fun transpose(): Move {
        return Move(e2, e1)
    }
}

private class Cactus(val n: Int, edges: Array<Pair<Int, Int>>) {
    val adjList = Array(n) { mutableSetOf<Int>() }

    init {
        for (edge in edges) {
            adjList[edge.first].add(edge.second)
            adjList[edge.second].add(edge.first)
        }
    }

    private fun findPath(start: Int, v: Int, finish: Int, path: MutableList<Int>, used: BooleanArray): Boolean {
        path.add(v)
        if (v == finish) {
            return true
        }
        used[v] = true
        for (u in adjList[v]) {
            if (u == finish && v == start) continue
            if (used[u]) continue
            if (findPath(start, u, finish, path, used)) {
                return true
            }
        }
        path.removeLast()
        return false
    }

    private fun isConnected(): Boolean {
        val used = BooleanArray(n)
        findPath(0, 0, n, mutableListOf(), used)
        return used.all { it }
    }

    private fun makeMove(moves: MutableList<Move>, e1: Pair<Int, Int>, e2: Pair<Int, Int>) {
        moves.add(Move(e1, e2))
        adjList[e1.first].remove(e1.second)
        adjList[e1.second].remove(e1.first)
        adjList[e2.first].add(e2.second)
        adjList[e2.second].add(e2.first)
        check(isConnected())
    }

    private fun makeAllCyclesLength3(moves: MutableList<Move>) {
        val initialEdges = buildList {
            for (v in 0..<n) {
                for (u in adjList[v]) {
                    if (v < u) {
                        add(v to u)
                    }
                }
            }
        }
        val path = mutableListOf<Int>()
        val used = BooleanArray(n)
        for ((v, u) in initialEdges) {
            path.clear()
            used.fill(false)
            if (findPath(v, v, u, path, used) && path.size > 3) {
                makeMove(moves, v to u, v to path[2])
            }
        }
    }

    private fun calcUpAndDepth(
        v: Int,
        parent: Int,
        d: Int,
        depth: IntArray,
        up: IntArray,
        bridges: MutableList<Pair<Int, Int>>
    ) {
        depth[v] = d
        up[v] = depth[v]
        for (to in adjList[v]) {
            if (to == parent) continue
            if (depth[to] >= 0) {
                up[v] = minOf(up[v], depth[to])
            } else {
                calcUpAndDepth(to, v, d + 1, depth, up, bridges)
                if (up[to] > depth[v]) {
                    bridges.add(v to to)
                }
                up[v] = minOf(up[v], up[to])
            }
        }
    }

    private fun getAllBridges(): List<Pair<Int, Int>> {
        val depth = IntArray(n) { -1 }
        val up = IntArray(n)
        val bridges = mutableListOf<Pair<Int, Int>>()
        calcUpAndDepth(0, -1, 0, depth, up, bridges)
        return bridges
    }

    private fun connectBridgesToFirst(moves: MutableList<Move>) {
        val bridges = getAllBridges()
        for ((from, to) in bridges) {
            if (from == 0) continue
            check(to != 0)
            makeMove(moves, from to to, 0 to to)
        }
    }

    private fun connectAllToFirst(moves: MutableList<Move>) {
        val bridges = mutableListOf<Int>()
        val leaves = mutableListOf<Int>()
        for (v in adjList[0]) {
            if (adjList[v].size == 1) {
                leaves.add(v)
                continue
            }
            var isBridge = true
            for (u in adjList[v]) {
                if (u == 0) continue
                if (adjList[u].contains(0)) {
                    isBridge = false
                    break
                }
            }
            if (isBridge) {
                bridges.add(v)
            }
        }
        handleAllBridgesToFirst(bridges, leaves, moves)
        while (true) {
            var changed = false;
            for (pv in adjList[0]) {
                val v = adjList[pv].find { it != 0 && !adjList[it].contains(0) } ?: continue
                val u = adjList[v].find { it != pv && adjList[it].contains(pv) }!!
                val leaf = leaves.removeLast()
                makeMove(moves, 0 to leaf, u to leaf)
                makeMove(moves, pv to u, v to leaf)
                makeMove(moves, pv to v, 0 to v)
                bridges.add(v)
                handleAllBridgesToFirst(bridges, leaves, moves)
                changed = true
                break
            }
            if (!changed) {
                break
            }
        }
        for (x in 1..<n) {
            check(adjList[x].contains(0))
            check(adjList[x].size <= 2)
        }
    }

    private fun handleAllBridgesToFirst(bridges: MutableList<Int>, leaves: MutableList<Int>, moves: MutableList<Move>) {
        while (bridges.isNotEmpty()) {
            val v = bridges.removeLast()
            val u = adjList[v].find { it != 0 }!!
            val w = adjList[u].find { it != v && adjList[it].contains(v) }!!
            makeMove(moves, u to w, 0 to u)
            makeMove(moves, v to w, 0 to w)
            if (adjList[w].size == 1) {
                leaves.add(w)
            } else {
                bridges.add(w)
            }
        }
    }

    private fun sortCycles(moves: MutableList<Move>) {
        val pairs = IntArray(n) { -1 }
        for (v in 1..<n) {
            for (u in adjList[v]) {
                if (u != 0) {
                    pairs[v] = u
                }
            }
        }
        val countPairs = pairs.count { it >= 0 } / 2
        for (id in 0..<countPairs) {
            val needV = 1 + 2 * id
            val needU = needV + 1
            if (pairs[needV] == needU) continue
            if (pairs[needV] < 0 && pairs[needU] < 0) {
                val findAnyWithPair = (needU + 1..<n).find { pairs[it] >= 0 }!!
                makeMoveWithPairs(moves, pairs, findAnyWithPair to pairs[findAnyWithPair], needV to needU)
            } else if (pairs[needV] < 0) {
                makeMoveWithPairs(moves, pairs, needU to pairs[needU], needV to needU)
            } else if (pairs[needU] < 0) {
                makeMoveWithPairs(moves, pairs, needV to pairs[needV], needV to needU)
            } else {
                val anyNoPair = (needU + 1..<n).find { pairs[it] < 0 }!!
                makeMoveWithPairs(moves, pairs, needV to pairs[needV], pairs[needV] to anyNoPair)
                makeMoveWithPairs(moves, pairs, needU to pairs[needU], needV to needU)
            }
        }
    }

    private fun makeMoveWithPairs(moves: MutableList<Move>, pairs: IntArray, e1: Pair<Int, Int>, e2: Pair<Int, Int>) {
        makeMove(moves, e1, e2)
        pairs[e1.first] = -1
        pairs[e1.second] = -1
        pairs[e2.first] = e2.second
        pairs[e2.second] = e2.first
    }

    fun makeCanonical(): List<Move> {
        val moves = mutableListOf<Move>()
        makeAllCyclesLength3(moves)
        connectBridgesToFirst(moves)
        connectAllToFirst(moves)
        sortCycles(moves)
        return moves
    }

    fun isTheSameAs(other: Cactus): Boolean {
        return adjList.contentEquals(other.adjList)
    }
}

private fun solve(g1: Cactus, g2: Cactus, n: Int, m: Int): List<Move>? {
    if (g1.isTheSameAs(g2)) {
        return emptyList()
    }
    if (n % 2 == 1 && m == ((n - 1) / 2) * 3) {
        return null
    }
    val answer = buildList {
        addAll(g1.makeCanonical())
        addAll(g2.makeCanonical().reversed().map { it.transpose() })
    }
    check(g1.isTheSameAs(g2))
    return answer
}

private fun readGraph(n: Int, m: Int): Cactus {
    val edges = Array(m) {
        val (v, u) = readInts()
        (v - 1) to (u - 1)
    }
    return Cactus(n, edges)
}

fun main() {
    val (n, m) = readInts()
    val g1 = readGraph(n, m)
    val g2 = readGraph(n, m)
    val answer = solve(g1, g2, n, m)
    if (answer == null) {
        println("NO")
    } else {
        println("YES")
        println(answer.size)
        answer.forEach {
            println("${it.e1.first + 1} ${it.e1.second + 1} ${it.e2.first + 1} ${it.e2.second + 1}")
        }
    }
}

private fun readInts() = readln().split(' ').map { it.toInt() }