import java.io.File
import java.io.PrintWriter
import java.lang.IllegalArgumentException
import kotlin.random.Random

data class Test(val array: IntArray, val requests: Array<Pair<Int, Int>>)

object Util {
    val maxC = 1e9.toInt()
    val maxN = 2e5.toInt()
    val maxQ = 2e5.toInt()
    val random = Random(123)

    fun randomPair(n: Int): Pair<Int, Int> {
        val a = random.nextInt(n)
        val b = random.nextInt(n)
        return Pair(Math.min(a, b), Math.max(a, b))
    }

    fun collectQ(s: Iterator<Pair<Int, Int>>, q: Int): Array<Pair<Int, Int>> {
        return s.asSequence().take(q).toList().toTypedArray()
    }
}

fun sampleTest1(): Test = Test(
    intArrayOf(3, 2, 1, 2, 3),
    arrayOf(Pair(0, 4), Pair(1, 3), Pair(0, 1), Pair(4, 4))
)

fun sampleTest2(): Test = Test(
    intArrayOf(1, Util.maxC, 1, 1, 1, Util.maxC, 1),
    arrayOf(Pair(0, 2), Pair(3, 4), Pair(4, 6))
)

fun minTest(): Test = Test(
    intArrayOf(1),
    arrayOf(Pair(0, 0))
)

// seems to require @Experimental declaration in Kotlin version Polygon uses.
@ExperimentalStdlibApi
fun augmentWithUndo(gen: Iterator<List<Pair<Int, Int>>>,
                    n: Int,
                    noiseGen: Iterator<List<Pair<Int, Int>>>? = null): Iterator<Pair<Int, Int>> = iterator {
    gen.forEach { lst ->
        lst.forEach { yield(it) }

        lst.shuffled(Util.random).forEach {
            if (it.first != 0)
                yield(Pair(0, it.first - 1))
            if (it.second != n - 1)
                yield(Pair(it.second + 1, n - 1))
        }

        noiseGen?.next()?.forEach {
            yield(it)
        }
    }
}

@ExperimentalStdlibApi
fun mixWithPyramidise(test: Test, perBlock: Int, noisePerBlock: Int, q: Int): Test {
    val n = test.array.size

    fun rawGen() = iterator {
        while (true) {
            val ends = (0 until 2*perBlock).map {Util.random.nextInt(n)}.toTypedArray()
            ends.sort()

            yield((0 until perBlock).map {Pair(ends[it], ends[ends.size - 1 - it]) })
        }
    }

    fun noiseGen() = iterator {
        while (true) {
            yield((0 until noisePerBlock).map {Util.randomPair(n)})
        }
    }

    fun gen() = augmentWithUndo(rawGen(), n, noiseGen())

    return Test(test.array, Util.collectQ(gen(), q))
}

@ExperimentalStdlibApi
fun stressTest(n: Int, perBlock: Int): Test {
    val arr = IntArray(n) { 1 }
    fun rawGen() = iterator {
        while (true) {
            yield((0 until perBlock).map { _ -> Util.randomPair(n) }.toList())
        }
    }

    fun gen() = augmentWithUndo(rawGen(), n)

    return Test(arr, Util.collectQ(gen(), Util.maxQ))
}

@ExperimentalStdlibApi
fun pyramidise(n: Int, perBlock: Int, noisePerBlock: Int): Test {
    val arr = IntArray(n) { 1 }
    return mixWithPyramidise(Test(arr, arrayOf()), perBlock, noisePerBlock, Util.maxQ)
}

fun mixWithRandomQueries(t: Test, q: Int): Test = Test(
    t.array,
    (1..q).map { Util.randomPair(t.array.size) }.toTypedArray()
)

fun pureRandom(n: Int, initC: Int): Test {
    val arr = IntArray(n) { Util.random.nextInt(1, initC + 1) }
    return mixWithRandomQueries(Test(arr, arrayOf()), Util.maxQ)
}

fun simplePyramid(n: Int): Test {
    val arr = IntArray(n) { Math.min(it + 1, n - it) }
    return mixWithRandomQueries(Test(arr, arrayOf()), Util.maxQ)
}

fun reversePyramid(n: Int): Test {
    val arr = IntArray(n) { Math.max(it + 1, n - it) }
    return mixWithRandomQueries(Test(arr, arrayOf()), Util.maxQ)
}

fun hugeAnswer(n: Int): Test {
    val arr = IntArray(n) { if (Math.max(it + 1, n - it) == n) Util.maxC else 1 }
    return mixWithRandomQueries(Test(arr, arrayOf()), Util.maxQ)
}

fun simpleIncrSequence(n: Int): Test {
    val arr = IntArray(n) { it + 1 }
    return mixWithRandomQueries(Test(arr, arrayOf()), Util.maxQ)
}

@ExperimentalStdlibApi
fun simpleIncrPyramidise(n: Int, perBlock: Int, noisePerBlock: Int): Test {
    val arr = IntArray(n) { it + 1 }
    return mixWithPyramidise(Test(arr, arrayOf()), perBlock, noisePerBlock, Util.maxQ)
}

@ExperimentalStdlibApi
val tests: Array<() -> Test> = arrayOf(
    ::sampleTest1,
    ::sampleTest2,
    ::minTest,
    { -> stressTest(5, 20) },
    { -> stressTest(8, 25) },
    { -> stressTest(10, 30) },
    { -> pureRandom(1000, 1) },
    { -> pureRandom(Util.maxN, 1)},
    { -> pureRandom(Util.maxN, Util.maxC)},
    { -> simplePyramid(Util.maxN) },
    { -> reversePyramid(Util.maxN) },
    { -> hugeAnswer(Util.maxN) },
    { -> stressTest(Util.maxN, 2000) },
    { -> pyramidise(Util.maxN, 1000, 10) },
    { -> pyramidise(Util.maxN, 10000, 10) },
    { -> simpleIncrSequence(Util.maxN) },
    { -> simpleIncrPyramidise(Util.maxN, 1000, 10) },
    { -> simpleIncrPyramidise(Util.maxN, 10000, 10) },
)

fun writeTest(t: Test, stream: PrintWriter) {
    stream.println("${t.array.size} ${t.requests.size}")
    stream.println(t.array.joinToString(" "))
    t.requests.forEach {
        stream.println("${it.first + 1} ${it.second + 1}")
    }
}

@ExperimentalStdlibApi
fun main(args: Array<String>) {
    if (args.isNotEmpty()) {
        if (args.size != 1) {
            throw IllegalArgumentException()
        }

        val num = args[0].toInt() - 1
        val test = tests[num]()
        writeTest(test, PrintWriter(System.out))
    } else {
        for (e in tests.withIndex()) {
            System.err.println("Generating test ${e.index + 1}")
            File("${e.index + 1}").bufferedWriter().use { out ->
                writeTest(e.value(), PrintWriter(out))
            }
        }
    }
}
